<?php
//
// Copyright 2015-2019, Julian Catchen <jcatchen@illinois.edu>
//
// This file is part of Chromonomer.
//
// Chromonomer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Chromonomer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Chromonomer.  If not, see <http://www.gnu.org/licenses/>.
//
include("header.php");

$lg  = isset($_GET['lg']) ? $_GET['lg'] : 0;
$ver = isset($_GET['v'])  ? $_GET['v']  : "";

write_extended_header($lg);

//
// Check if JSON files are present, if not, revert back to textual display.
//
$json_exists = false;
if (file_exists("$file_path/${ver}/${lg}_after.json"))
    $json_exists = true;

echo <<< EOQ

<div class="body">
<h1 class="lg">$lg</h1>
<h5 class="lg"><a href="$root_path/index.php?v=${ver}"><i class="fa fa-arrow-circle-left"></i> Return to Assembly Summary.</a></h5>

EOQ;

if ($json_exists) {
  echo <<< EOQ

<div id="size_slider_div" class="slider">
  <p>Image size: <input id="size_slider" type="range" min="100" max="10000" step="100" />
  <span id="size_display"></span></p>
</div>

<table class="lg">
<tr>
  <th>Before Chromonomer Integration</th>
  <th>After Chromonomer Integration</th>
</tr>
<tr>
    <td>
    <a id="pdfcnv_before" style="float: left; padding-left: 0.5em; padding-top: 0.5em;"><i class="menu_icon fa fa-file-pdf-o fa-lg" title="Click to download a PDF of this plot."></i></a>
    <svg id="lg_before" class="lg_before"></svg>
    <div class="txt_link"><a href="$root_path/${ver}/${lg}_before.php">View textual version.</a></div>
    </td>
    <td>
    <a id="pdfcnv_after" style="float: left; padding-left: 0.5em; padding-top: 0.5em;"><i class="menu_icon fa fa-file-pdf-o fa-lg" title="Click to download a PDF of this plot."></i></a>
    <svg id="lg_after" class="lg_after"></svg>
    <div class="txt_link"><a href="$root_path/${ver}/${lg}_after.php">View textual version.</a></div>
  </td>
</tr>
</table>

EOQ;
} else {
  echo <<< EOQ
<table class="lg">
<tr>
  <th>Before</th>
  <th>After</th>
</tr>
<tr>
  <td>
  <iframe id="before_iframe" 
          scrolling="no" 
          onload="this.style.height = (this.contentWindow.document.body.offsetHeight+25) + 'px';"
          frameborder="0" 
          src="$root_path/${ver}/${lg}_before.php"></iframe>
  </td>
  <td>
    <iframe id="after_iframe" 
          scrolling="no" 
          onload="this.style.height = (this.contentWindow.document.body.offsetHeight+25) + 'px';"
          frameborder="0" 
          src="$root_path/${ver}/${lg}_after.php"></iframe>
  </td>
</tr>
</table>

EOQ;
}

echo <<< EOQ
<div id="test_label"></div>
    
<div class="assembly">
<a name="final_assembly"></a>
<h2 class="head">Final Assembly</h2>

EOQ;

write_assembly($lg, $ver);

echo <<< EOQ
</div>

<h5 class="lg"><a href="$root_path/index.php?v=${ver}"><i class="fa fa-arrow-circle-left"></i> Return to Assembly Summary.</a></h5>
    
<div class="footer">
  <div style="float: left;">
      Last Updated: 
EOQ;

    include("last_modified.php");

    echo <<< EOQ
  </div>
</div>
</div>

<form id="pdfform" name="pdfform" method="post" action="$root_path/svg2pdf.php">
<input type="hidden" name="data" id="svgdata" value="" />
</form>

</body>

<script type="text/javascript">

$(document).ready(function() {
  //
  // Make the AJAX query for JSON encoded data describing the 'before' and 'after' linkage groups.
  //
  var url = "$root_path/${ver}/${lg}_before.json";
  $.ajax({
    dataType: "json",
    url: url,
    success: ajax_fetch_lg_before
  });
  url = "$root_path/${ver}/${lg}_after.json";
  $.ajax({
    dataType: "json",
    url: url,
    success: ajax_fetch_lg_after
  });

  //
  // Set up the image size selector.
  //
  $("#size_display").html(lg_config.height + "px");
  $("#size_slider").on("change", function() {
      lg_config.height = $("#size_slider").val();
      $("#size_display").html(lg_config.height + "px");
      redraw_lgs();
  });

  $("#pdfcnv_before").on("click", {type: "before"}, svg2pdf);
  $("#pdfcnv_after").on("click", {type: "after"}, svg2pdf);
});
</script>
</html>

EOQ;
?>
