var lg_config = {
    width:           575,
    height:          700,
    margin:          {top: 5, right: 5, bottom: 5, left: 5},
    lg_width:        25,
    lg_margin:       10,
    min_scaf_height: 20,
    scaf_start:      225,
    scaf_width:      250,
    scaf_margin:     2,
    scaf_font_size:  10,
    scaf_colors:     {},
    mb_scale_x:      480
};

function ajax_fetch_lg_before(json, status, jqXHR)
{
    //
    // Remove the existing SVG elements.
    //
    var dp = document.getElementById('lg_before');
    while (dp.firstChild) {
        dp.removeChild(dp.firstChild);
    }

    draw_lg("lg_before", json);

    lg_config.json_before = json;
}

function ajax_fetch_lg_after(json, status, jqXHR)
{
    var dp = document.getElementById('lg_after');
    while (dp.firstChild) {
        dp.removeChild(dp.firstChild);
    }

    draw_lg("lg_after", json);

    lg_config.json_after = json;
}

function redraw_lgs()
{
    //
    // Remove the existing SVG elements.
    //
    var dp = document.getElementById('lg_before');
    while (dp.firstChild) {
	dp.removeChild(dp.firstChild);
    }
    draw_lg("lg_before", lg_config.json_before);

    var dp = document.getElementById('lg_after');
    while (dp.firstChild) {
	dp.removeChild(dp.firstChild);
    }
    draw_lg("lg_after", lg_config.json_after);
}

function draw_lg(dom_id, json)
{
    var margin = lg_config.margin;

    //
    // Determine the total length of the scaffolds to scale the image height.
    //
    var scaffolds  = [];
    var k          = 1;
    var tot_len_bp = 0;
    var scaf_len   = 0;
    for (var i = 0; i < json.nodes.length; i++)
	for (var j = 0; j < json.nodes[i].scaffolds.length; j++) {
	    scaf_len    = +json.nodes[i].scaffolds[j].e_bp - +json.nodes[i].scaffolds[j].s_bp + 1;
	    scaffolds.push({'i':    k,
			    'len':  scaf_len,
			    'id':   json.nodes[i].scaffolds[j].id,
			    's_bp': tot_len_bp + 1,
			    'e_bp': tot_len_bp + scaf_len,
			    'ori':  json.nodes[i].scaffolds[j].ori,
			    'log': +json.nodes[i].scaffolds[j].log,
			    'old':  json.nodes[i].scaffolds[j].old,
			    'node': i});
	    tot_len_bp += scaf_len;
	    k++;
	}

    var bp_px_ratio = lg_config.height / tot_len_bp;

    for (var i = 0; i < scaffolds.length; i++)
	scaffolds[i].px = scaffolds[i].len * bp_px_ratio;

    scaffolds.sort(function(a, b) {
	return a.len < b.len ? -1 : (a.len > b.len ? 1 : 0);
    });

    // console.log("Len Sorted Scaffolds: ");
    // for (var i = 0; i < scaffolds.length; i++)
    // 	console.log("Scaffold: " + scaffolds[i].id + ", Len: " + scaffolds[i].len + ", Px: " + scaffolds[i].px);

    //
    // Adjust the lengths of the scaffolds (in pixels) to bring the smallest scaffolds
    // up to a minimum height by taking pixels from the largest scaffolds.
    //
    var end     = false;
    var i       = 0
    var j       = scaffolds.length - 1;
    
    while (end == false) {
    	if ((scaffolds[j].px - 4) < lg_config.min_scaf_height)
    	    j = scaffolds.length - 1;
	else
	    i = 0;

    	while (i < scaffolds.length &&
    	       j >= 0 &&
    	       scaffolds[i].px < lg_config.min_scaf_height) {
	    
    	    if ((scaffolds[j].px - 4) < lg_config.min_scaf_height) {
		if ((scaffolds[scaffolds.length - 1].px - 4) < lg_config.min_scaf_height)
    		    end = true;
    		break;
    	    }

    	    scaffolds[i].px += 4;
    	    scaffolds[j].px -= 4;
	    // console.log("     Removing pixels from " + scaffolds[j].id + "[" + j + "]" + " and adding them to " + scaffolds[i].id + "[" + i + "]");
    	    i++;
    	    j--;
    	}

    	if (end == false && scaffolds[0].px > lg_config.min_scaf_height)
    	    end = true;
    }

    // console.log("Px Adjusted Scaffolds: ");
    // for (var i = 0; i < scaffolds.length; i++)
    // 	console.log("Scaffold: " + scaffolds[i].id + ", Len: " + scaffolds[i].len + ", Px: " + scaffolds[i].px);

    //
    // Re-sort scaffolds to their order of occurance.
    // Assign X/Y coordinates to scaffolds and calculate total image length.
    //
    scaffolds.sort(function(a, b) {
    	return a.i < b.i ? -1 : (a.i > b.i ? 1 : 0);
    });

    var px = lg_config.lg_margin;
    for (var i = 0; i < scaffolds.length; i++) {
    	scaffolds[i].y = px;
    	px += Math.ceil(scaffolds[i].px);
    	px += lg_config.scaf_margin;
    }

    //
    // Assign scaffold colors and generate scaffold labels.
    //
    var c           = d3.scale.category20c();
    var index       = 0;
    var max_adj     = 0;
    for (var i = 0; i < scaffolds.length; i++) {
	if (!(scaffolds[i].id in lg_config.scaf_colors)) {
	    lg_config.scaf_colors[scaffolds[i].id] = c(index % 20);
	    index++;
	}

	scaffolds[i].clr        = lg_config.scaf_colors[scaffolds[i].id];
	scaffolds[i].label      = generate_scaffold_label(json.path, scaffolds[i]);
	scaffolds[i].prev_label = generate_old_scaffolds_label(json.path, scaffolds[i]);
	scaffolds[i].class_id   = scaffolds[i].id.replace(".", "_");

	//
	// Measure the height required to display the text on this node.
	//
	scaffolds[i].height = measure_scaffold(scaffolds[i]);
	max_adj = (scaffolds[i].height - scaffolds[i].px) > max_adj ? (scaffolds[i].height - scaffolds[i].px) : max_adj;
    }

    //
    // Adjust the margin if necesary to accommodate expanding scaffolds.
    //
    if (margin.bottom < max_adj)
	margin.bottom = max_adj;

    //
    // Set the image size based on the window size.
    //
    var width  = lg_config.width;
    var height = px + lg_config.lg_margin + margin.top + margin.bottom;

    //
    // For each node in the linkage map, determine the cM, basepair, and pixel boundaries.
    //
    var nodes = [];
    var index = 0;
    var first, last;
    for (var i = 0; i < json.nodes.length; i++) {
    	//
    	// Look up the first and last scaffolds in this node to get the basepair and pixel boundaries.
    	//
    	first  = index;
    	last   = index + json.nodes[i].scaffolds.length - 1;

    	nodes.push({'i':    i,
    		    's_bp': scaffolds[first].s_bp,
    		    's_px': scaffolds[first].y,
    		    'e_bp': scaffolds[last].e_bp,
    		    'e_px': scaffolds[last].y + scaffolds[last].px,
		    'cMs':  []});
    	index += json.nodes[i].scaffolds.length;
    }
    
    var cur_node  = 0;
    var next_node = cur_node + 1;
    var last_cM, end_cM;

    nodes[cur_node].s_cM = +json.nodes[cur_node].cMs[0];

    while (next_node < nodes.length) {
    	//
    	// If we share a centiMorgan label with the next node, this is our end boundary.
    	// Otherwise, interpolate between the two labels.
    	//
    	last_cM = json.nodes[cur_node].cMs.length - 1;

        if (json.nodes[next_node].cMs.length > 1) {
    	    end_cM  = json.nodes[cur_node].cMs[last_cM] == json.nodes[next_node].cMs[0] ?
    		(+json.nodes[cur_node].cMs[last_cM] + +json.nodes[next_node].cMs[1]) / 2.0 :
    		(+json.nodes[cur_node].cMs[last_cM] + +json.nodes[next_node].cMs[0]) / 2.0;
	} else {
    	    end_cM  = json.nodes[cur_node].cMs[last_cM] == json.nodes[next_node].cMs[0] ?
    		(+json.nodes[cur_node].cMs[last_cM - 1] + +json.nodes[next_node].cMs[0]) / 2.0 :
    		(+json.nodes[cur_node].cMs[last_cM] + +json.nodes[next_node].cMs[0]) / 2.0;
 	}

        nodes[cur_node].e_cM  = end_cM;
    	nodes[next_node].s_cM = end_cM;
    	cur_node++;
    	next_node++;
    }
    nodes[cur_node].e_cM = +json.nodes[cur_node].cMs[json.nodes[cur_node].cMs.length - 1];

    // console.log(nodes)

    //
    // Add the cM labels ot the appropriate nodes.
    //
    var prev_node, prev_cM, next_cM;
    var printed = {};

    for (cur_node = 0; cur_node < nodes.length; cur_node++) {
	if (json.nodes[cur_node].cMs.length == 1) {
	    nodes[cur_node].cMs.push(json.nodes[cur_node].cMs[0]);
	    printed[json.nodes[cur_node].cMs[0]] = 1;
	}
    }
    for (cur_node = 0; cur_node < nodes.length; cur_node++) {
	for (var i = 0; i < json.nodes[cur_node].cMs.length; i++) {
	    if (!(json.nodes[cur_node].cMs[i] in printed)) {
		nodes[cur_node].cMs.push(json.nodes[cur_node].cMs[i]);
		printed[json.nodes[cur_node].cMs[i]] = 1;
	    }
	}
    }

    //
    // Assign pixel positions to the centiMorgan labels.
    //
    var cM_tics = [];
    var cM_span, cM_px, cM_ratio, px_span, px;

    for (var i = 0; i < nodes.length; i++) {
	cM_span = nodes[i].e_cM - nodes[i].s_cM;
	px_span = nodes[i].e_px - nodes[i].s_px

	for (var j = 0; j < nodes[i].cMs.length; j++) {
	    cM_dist  = +nodes[i].cMs[j] - nodes[i].s_cM;
	    cM_ratio = cM_dist / cM_span;
	    px       = nodes[i].s_px + (px_span * cM_ratio);

	    cM_tics.push({'label': nodes[i].cMs[j], 'px': px});
	}
    }

    //
    // Convert our scaffold and cM arrays to be dictionaries.
    //
    var cM_dict = {};
    for (var i = 0; i < cM_tics.length; i++)
	cM_dict[cM_tics[i].label] = cM_tics[i];
    var scaf_dict = {};
    for (var i = 0; i < scaffolds.length; i++)
	scaf_dict[scaffolds[i].id + scaffolds[i].node] = scaffolds[i];

    //
    // Assign pixel coordinates for marker genes.
    //
    var markers = [];
    var id, bp_span, bp_dist, bp_ratio;
    for (var i = 0; i < json.markers.length; i++) {
    	id       = json.markers[i].s + json.markers[i].node;

	if (!(id in scaf_dict)) {
    	    console.log("scaffold " + id + " is not in dictionary.");
	    continue;
	}

    	px_span  = scaf_dict[id].px;
    	bp_span  = scaf_dict[id].e_bp - scaf_dict[id].s_bp + 1;
    	bp_dist  = json.markers[i].bp;
    	bp_ratio = bp_dist / bp_span;

	if (scaf_dict[id].ori == "Reverse")
	    bp_ratio = 1 - bp_ratio;
	
    	px = scaf_dict[id].y + (bp_ratio * px_span);

    	if (!(json.markers[i].cM in cM_dict)) {
    	    console.log(json.markers[i].cM + " is not in dictionary.");
	    continue;
	}
	
    	markers.push({'id':      json.markers[i].id,
    		      'scaf_px': px,
    		      'cM_px':   cM_dict[json.markers[i].cM].px});
    }
    
    //
    // Adjust the image height, depending on the number of chromosomes being displayed.
    //
    var calc_height = lg_config.min_row_height * lg_config.max_chrs;
    height = calc_height > height ? calc_height : height;

    width  = width  - margin.left - margin.right;
    height = height - margin.top  - margin.bottom;

    var lg = d3.select("#" + dom_id)
	.attr("width", width + margin.left + margin.right)
	.attr("height", height + margin.top + margin.bottom)
	.attr("viewBox", "0 0 " + (width + margin.left + margin.right) + " " + (height + margin.top + margin.bottom))
	.attr("preserveAspectRatio", "xMinYMin meet")
	.append("g")
	.attr("transform", "translate(" + margin.left + "," + margin.top + ")");

    //
    // Draw the scaffolds.
    //
    var scaf_groups = lg.selectAll("g")
	.data(scaffolds)
	.enter().append("g")
	.attr("class", "scaf_group")
    	.attr("transform", function(d) { return "translate(" + lg_config.scaf_start + "," + d.y + ")";});

    scaf_groups.append("rect")
        .attr("class", function(d) {
	    var cls = "lg_scaf " + d.class_id;
	    if (d.old.length > 0)
		cls += " " + d.old[0].replace(".", "_");
	    return cls;
	})
        .attr("x", 0)
        .attr("y", 0)
        .attr("rx", 3)
        .attr("ry", 3)
        .attr("width", lg_config.scaf_width)
        .attr("height", function(d) { return d.px; })
	.style("fill-opacity", "0.5")
        .style("fill",   function(d) { return d.clr; });

    scaf_groups.append("foreignObject")
        .attr("class", function(d) { return "fo " + d.class_id; })
        .attr("width", lg_config.scaf_width)
        .attr("height", function(d) { return d.px; })
	.on("mouseover", function(d){
	    var cls = d.class_id;
	    cls += d.old.length > 0 ? ",." + d.old[0].replace(".", "_") : "";
	    t = d3.selectAll("." + cls).style("fill-opacity", "1.0").style("stroke", "#888");
	    expand_retile_scaffolds(dom_id, d);
	})
    	.on("mouseout", function(d){
	    var cls = d.class_id;
	    cls += d.old.length > 0 ? ",." + d.old[0].replace(".", "_") : "";
	    d3.selectAll("." + cls).style("fill-opacity", "0.5").style("stroke", "#000");
	    collapse_retile_scaffolds(dom_id, d);
	})
        .append("xhtml:div")
    	.attr("class", "scaffold_label")
	.html(function(d) { return d.label; })
	.append("xhtml:div")
    	.attr("class", "prev_scaffold_label")
	.html(function(d) { return d.prev_label; });
    
    //
    // Draw the linkage group.
    //
    var lg_obj = lg.append("g")
	.attr("class", "lg_obj");
    lg_obj.append("rect")
	.attr("class", "lg")
        .attr("x", 75)
        .attr("y", 0)
        .attr("rx", 10)
        .attr("ry", 10)
        .attr("width", lg_config.lg_width)
        .attr("height", height);

    //
    // We want to pad the top and bottom of the linkage group to avoid drawing too near the 
    // curved sections.
    //
    var lg_height = height - (2 * lg_config.lg_margin);

    lg_obj.selectAll("line")
        .data(cM_tics)
	.enter().append("line")
	.attr("class", "lg_tic")
        .attr("x1", function(d) { return 75; })
        .attr("x2", function(d) { return 75 + lg_config.lg_width; })
        .attr("y1", function(d) { return d.px; })
	.attr("y2", function(d) { return d.px; });

    lg_obj.selectAll("text")
	.data(cM_tics)
	.enter().append("text")
    	.attr("class", "lg_label")
        .attr("x", 70)
        .attr("y", function(d) { return d.px; })
	.attr("dominant-baseline", "central")
        .text(function(d) { return d.label; });

    //
    // Draw the markers.
    //
    lg_obj = lg.append("g")
	.attr("class", "markers_group");
    lg_obj.selectAll("line")
	.data(markers)
	.enter().append("line")
	.attr("class", "marker")
	.attr("id", function(d) { return d.id; })
        .attr("x1", 75 + lg_config.lg_width)
        .attr("x2", lg_config.scaf_start)
        .attr("y1", function(d) { return d.cM_px; })
        .attr("y2", function(d) { return d.scaf_px; })
    	.style("stroke", "#000")
    	.style("stroke-width", "1");

    //
    // Generate a Megabase scale based on the scaffolds.
    //
    var mb_tics = [];
    var mb_tic  = 5000000;
    var mb      = mb_tic;
    var mb_ratio;
    var mb_px;
    for (var i = 0; i < scaffolds.length; i++) {

	if (scaffolds[i].s_bp < mb && scaffolds[i].e_bp > mb) {

	    while (mb < scaffolds[i].e_bp) {
		mb_ratio = (mb - scaffolds[i].s_bp) / (scaffolds[i].e_bp - scaffolds[i].s_bp + 1);
		mb_px    = scaffolds[i].y + (mb_ratio * scaffolds[i].px);
		mb_tics.push({label: (Math.floor(mb / 1000000)) + "Mb", px: mb_px });
		mb += mb_tic;
	    }
	}
    }
    var j = scaffolds.length - 1;
    mb_tics.push({label: (Math.floor((scaffolds[j].e_bp / 1000000) * 100) / 100) + "Mb", px: scaffolds[j].y + scaffolds[j].px });

    //
    // Draw the megabase scale.
    var scale = lg.append("g")
	.attr("class", "mb_scale");

    var mb_scale = scale.selectAll("g")
	.data(mb_tics)
	.enter().append("g")
	.append("line")
	.attr("x1", lg_config.mb_scale_x)
	.attr("x2", lg_config.mb_scale_x + 8)
    	.attr("y1", function(d) { return d.px; })
	.attr("y2", function(d) { return d.px; })
        .style("stroke", "#000")
    	.style("stroke-width", "2");
    scale.selectAll("g")
	.append("text")
    	.attr("class", "mb_label")
        .attr("x", lg_config.mb_scale_x + 10)
        .attr("y", function(d) { return d.px; })
	.attr("dominant-baseline", "central")
        .text(function(d) { return d.label; });
}

function expand_retile_scaffolds(dom_id, scaffold)
{
    if (scaffold.height <= scaffold.px)
	return;

    var lg = d3.select("." + dom_id);

    //
    // Expand the scaffold.
    //	
    lg.select("rect.lg_scaf." + scaffold.class_id).attr("height", scaffold.height);
    lg.select("foreignObject.fo." + scaffold.class_id).attr("height", scaffold.height);

    //
    // Move the scaffolds down.
    //
    var adj_px = scaffold.height - scaffold.px;

    lg.selectAll(".scaf_group")
	.attr("transform", function(c, i) {
	    y_px = i >= scaffold.i ?  c.y + adj_px : c.y;
	    c.y  = y_px;
	    return "translate(" + lg_config.scaf_start + "," + y_px + ")";
	});

    //
    // Move the markers down.
    //
    var m_bound = scaffold.y + scaffold.px;
    
    lg.selectAll(".marker")
	.attr("y2", function(c) {
	    return c.scaf_px > m_bound ? c.scaf_px + adj_px : c.scaf_px;
	});

    //
    // Move the megabase tics down.
    //
    lg.select("g.mb_scale").selectAll("line")
	.attr("y1", function(d) {
	    return d.px > m_bound ? d.px + adj_px : d.px;
	})
    	.attr("y2", function(d) {
	    return d.px > m_bound ? d.px + adj_px : d.px;
	});
    lg.select("g.mb_scale").selectAll("text")
	.attr("y", function(d) {
	    return d.px > m_bound ? d.px + adj_px : d.px;
	});
}

function collapse_retile_scaffolds(dom_id, scaffold)
{
    if (scaffold.height <= scaffold.px)
	return;

    var lg = d3.select("." + dom_id);
    
    lg.select("rect.lg_scaf." + scaffold.class_id).attr("height", scaffold.px);
    lg.select("foreignObject.fo." + scaffold.class_id).attr("height", scaffold.px);

    var adj_px = scaffold.height - scaffold.px;
    
    lg.selectAll(".scaf_group")
    	.attr("transform", function(d, i) {
    	    y_px = i >= scaffold.i ?  d.y - adj_px : d.y;
	    d.y  = y_px;
    	    return "translate(" + lg_config.scaf_start + "," + y_px + ")";
    	});

    var m_bound = scaffold.y + scaffold.px;
    
    lg.selectAll(".marker")
	.attr("y2", function(d) {
	    return d.scaf_px;
	});

    lg.select("g.mb_scale").selectAll("line")
	.attr("y1", function(d) {
	    return d.px;
	})
    	.attr("y2", function(d) {
	    return d.px;
	});
    lg.select("g.mb_scale").selectAll("text")
	.attr("y", function(d) {
	    return d.px;
	});
}

function generate_scaffold_label(path, scaffold)
{
    var len;
    var label;

    if (scaffold.log > 0)
	label = "<a href=\"" + path + scaffold.id + "\">" + scaffold.id + "</a>";
    else
	label = scaffold.id;

    label += " [";
    
    if (scaffold.len > 1000000) {
	len = scaffold.len / 1000000;
	len = Math.floor(len * 100) / 100;
	label += len + "Mb, ";
    } else {
	len = scaffold.len / 1000;
	len = Math.floor(len * 100) / 100;
	label += len + "Kb, ";
    }
    
    if (scaffold.ori == "Forward")
	label += "<span title=\"Forward orientation\"><i class=\"fa fa-arrow-right\"></i></span>]";
    else if (scaffold.ori == "Reverse")
	label += "<span title=\"Reverse orientation\"><i class=\"fa fa-arrow-left\"></i></span>]";
    else
	label += "<span title=\"Unknown orientation\"><i class=\"fa fa-question\"></i></span>]";

    return label;
}

function generate_old_scaffolds_label(path, scaffold)
{
    var label = "";

    if (scaffold.old.length > 0)
	label += "<p>Previous scaffolds:</p><ul>";

    for (var i = 0; i < scaffold.old.length; i++)
	label += "<li><a href=\"" + path + scaffold.old[i] + "\">" + scaffold.old[i] + "</a></li>";

    if (scaffold.old.length > 0)
	label += "</ul>";
    
    return label;
}

function measure_scaffold(scaffold) {

    var $div = $("<div class='scaffold_label' />")
    $div.append(scaffold.label);
    $div.append("<div class='prev_scaffold_label' />");
    $div.append(scaffold.prev_label);
    $("#test_label").append($div);
    
    var h = $div.height();
    
    $("#test_label").empty();
    
    return h;
}

function toggle_markers(id, root_path) {
    var div_obj = document.getElementById(id); 
    var caret   = document.getElementById(id + "_img");

    if (div_obj.style.display == "none") {
	div_obj.style.display = "";
	caret.src             = root_path + "/caret-d.png"; 
    }
    else {
	div_obj.style.display = "none";
	caret.src             = root_path + "/caret-u.png"; 
    }
}

function svg2pdf(event)
{
    var id  = "lg_" + event.data.type;
    var svg = document.getElementById(id);

    var svg_xml = (new XMLSerializer).serializeToString(svg);

    // console.log(svg_xml)
    
    var form = document.getElementById("pdfform");
    document.getElementById("svgdata").value = svg_xml;

    form.submit();

    return false;
}

function populate_chrr_runs(id, chrr_runs)
{
    var sel_run = (id == 1 ? sel_ver_1 : sel_ver_2);
    var subj_list =
        "<option value=\"--\">Select</option>\n";

    for (run in chrr_runs) {
        if (sel_run.length > 0 && sel_run == run)
            subj_list += "<option value=\"" + run + "\" selected=\"selected\">" + run + "</option>\n";
        else
            subj_list += "<option value=\"" + run + "\">" + run + "</option>\n";
    }

    $("#chrr_ver_" + id).html(subj_list);
    $("#chrr_ver_" + id).trigger("change");
    $("#chrr_lg_" + id).trigger("change")
}

function populate_chrr_run_lgs(event)
{
    var id        = "chrr_lg_" + event.data.type;
    var sel_lg    = (id == 1 ? sel_lg_1 : sel_lg_2);
    var subj_list =
        "<option value=\"--\">Select</option>\n";

    var run = document.getElementById("chrr_ver_" + event.data.type).value;
    var lgs = datasets[run];
    
    for (lg in lgs) {
        if (sel_lg.length > 0 && sel_lg == lgs[lg])
            subj_list += "<option value=\"" + lgs[lg] + "\" selected=\"selected\">" + lgs[lg] + "</option>\n";
        else
            subj_list += "<option value=\"" + lgs[lg] + "\">" + lgs[lg] + "</option>\n";
    }

    $("#"+id).html(subj_list);

    //
    // Clear any existing drawing.
    //
    id = event.data.type == "1" ? "lg_before" : "lg_after";
    var dp = document.getElementById(id);
    while (dp.firstChild) {
        dp.removeChild(dp.firstChild);
    }
}

function load_lg(event)
{
    var run = document.getElementById("chrr_ver_" + event.data.type).value;
    var lg  = document.getElementById("chrr_lg_" + event.data.type).value;

    var url = root_path + "/" + run + "/" + lg + "_after.json";
    
    console.log(url)
    if (event.data.type == "1") {
        $.ajax({
            dataType: "json",
            url: url,
            success: ajax_fetch_lg_before
        });
    } else {
        $.ajax({
            dataType: "json",
            url: url,
            success: ajax_fetch_lg_after
        });
    }

    //
    // Update the permalink.
    //
    permalink = build_url();
    $("#permalink").attr("href", permalink);
}

function build_url()
{
    var ver_1 = document.getElementById("chrr_ver_1").value;
    var lg_1  = document.getElementById("chrr_lg_1").value;
    var ver_2 = document.getElementById("chrr_ver_2").value;
    var lg_2  = document.getElementById("chrr_lg_2").value;
    
    var url = "/chromonomer/compare_lg.php";

    url += "?v1=" + ver_1;
    url += "&lg1=" + lg_1;
    url += "&v2=" + ver_2;
    url += "&lg2=" + lg_2;

    return url;
}
