// -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
//
// Copyright 2019, Julian Catchen <jcatchen@illinois.edu>
//
// This file is part of Chromonomer.
//
// Chromonomer is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Chromonomer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Chromonomer.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef __SYNTENY_H__
#define __SYNTENY_H__

#include <math.h>
#include <map>
using std::map;
#include <set>
using std::set;
#include <string>
using std::string;
#include <vector>
using std::vector;
#include <list>
using std::list;
#include <iomanip>
using std::setw;
using std::setprecision;
using std::fixed;

#include "constants.h"
#include "scaffold.h"
#include "linkage_map.h"
#include "input.h"

int      parse_gtf_file(string, vector<GeneAnn> &);
int      parse_gff_file(string, vector<GeneAnn> &);
int      parse_orthologs(string, map<string, Ortholog> &, map<string, string> &);
int      process_sort_orthologs(map<string, Ortholog> &, map<string, string> &,
                                vector<GeneAnn> &, vector<GeneAnn> &, map<string, vector<Ortholog>> &);
int      traverse_nodes_for_synteny(list<Node> &, ofstream &, map<string, vector<Ortholog>> &);
int      order_scaffolds_by_synteny(ostream &, vector<string> &, Node &, map<string, vector<Ortholog>> &);
int      orient_scaffolds_by_synteny(ostream &, string, Node &, map<string, vector<Ortholog>> &);
ori_type determine_scaffold_orientation_by_synteny(ostream &, vector<Ortholog> &);
int      prune_outoforder_synteny_genes(ostream &, ori_type, vector<Ortholog> &);
int      genes_in_path(vector<Ortholog> &, vector<int> &, vector<int> &, ori_type);
int      remove_anomolously_located_genes(ostream &, vector<Ortholog> &);
int      orient_orthologous_chromosome(ostream &, vector<string> &, set<string> &, map<string, vector<Ortholog>> &);
int      reverse_orth_gene_order(vector<Ortholog> &);

#endif // __SYNTENY_H__
